#############################################################################
# lu_plugin.tcl Copyright 2005, 2006, 2007 r0lZ
# PgcEdit plugin to manipulate menu's language units.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################
# Usage: copy this file in the <PgcEdit's install dir>/plugins directory.
# An "LU (Language Unit)" menu will be added in the Plugins menu.
# You need PgcEdit 0.6.1 or more to use this plugin.
#############################################################################
# Version History:
#
# v 1.0 (October 6, 2005)
# - First public release
# v 1.1 (May 7, 2006)
# - Added the "Clone current LU" function.
#############################################################################


# Add Plugins -> LU (Language Unit) submenu
if {! [winfo exists .mainmenu.plugins.lu]} {
	.mainmenu.plugins add cascade -label "LU (Language Unit)" -menu .mainmenu.plugins.lu
	menu .mainmenu.plugins.lu
	.mainmenu.plugins.lu add command -label "Change language code of current LU" -command {::plugins::lu::set_lang_code}
	.mainmenu.plugins.lu add command -label "Clone current LU" -command {::plugins::lu::clone_lu}
	.mainmenu.plugins.lu add separator
	.mainmenu.plugins.lu add command -label "About" -command {
			tk_messageBox  -title "LU (Language Unit) plugin" -icon info \
					-message "LU (Language Unit) plugin for PgcEdit 0.6.0 or more.\nVersion $::plugins::lu::version\nAuthor: r0lZ\nFreeware!"
		}
}


namespace eval ::plugins {}
namespace eval ::plugins::lu {

set version "1.1"

# Change the language code of the current LU
proc set_lang_code {} {
	if {![test_version 0.6.0 "LU (Language Unit) Plugin"]} return
	if {![info exists ::dvddir]} {bell; return}

	set type [lindex $::editor_cmds_from 0]
	if {$type != "menu"} {
		bell
		return
	}
	set indexes [split [lindex $::editor_cmds_from 1] ,]
	set vts [lindex $indexes 0]
	set lu  [lindex $indexes 1]
	set curcode $::menus($vts,$lu,lang)
	if {$curcode == "--"} {set curcode ""}

	set r [::stringreq::stringreq -window .lulangcode -title "Set LU language code" \
			-text "Curreent LU language code?\n(Use a blank string for 'undefined')" \
			-def $curcode -callback ::streams::validate_langcode -width 2 -geometry c \
			-cancelreturn "CANCELLED" -font $::config(fixed_width_font)]
	if {$r == "CANCELLED"} return
	if {$r == ""} {
		set code "\000\000"
		set text "--"
	} else {
		set code $r
		set text $r
	}
	set ::menus($vts,$lu,lang) $text
	set index [expr $lu * 8]
	if {$vts == 0} {
		set ::raw_sectors($vts,VMGM_PGCI_UT) \
			"[string range $::raw_sectors($vts,VMGM_PGCI_UT) 0 [expr $index-1]]$code[string range $::raw_sectors($vts,VMGM_PGCI_UT) [expr $index+2] end]"
	} else {
		set ::raw_sectors($vts,VTSM_PGCI_UT) \
			"[string range $::raw_sectors($vts,VTSM_PGCI_UT) 0 [expr $index-1]]$code[string range $::raw_sectors($vts,VTSM_PGCI_UT) [expr $index+2] end]"
	}
	::main::init_tree_menu_no_move
}

proc clone_lu {} {
	if {![test_version 0.6.0 "LU (Language Unit) Plugin"]} return
	if {![info exists ::dvddir]} {bell; return}

	set type [lindex $::editor_cmds_from 0]
	if {$type != "menu"} {
		bell
		return
	}
	set indexes [split [lindex $::editor_cmds_from 1] ,]
	set vts [lindex $indexes 0]
	set lu  [lindex $indexes 1]
	set newlu [expr $::menus($vts,numlus) + 1]
	set pgcvars [lsort -dictionary [array names ::menus $vts,$lu,*]]
	foreach v $pgcvars {
		set tmp [string range $v [expr [string first , $v]+1] end]
		set tmp [string range $tmp [expr [string first , $tmp]+1] end]
		set targetv $vts,$newlu,$tmp
		set ::menus($targetv) $::menus($v)
	}

	# rebuild VMGM/VTSM_PGCI_UT table
	if {$vts == 0} {set tablename VMGM_PGCI_UT} else {set tablename VTSM_PGCI_UT}
	set table $::raw_sectors($vts,$tablename)

	# new header, with new number of LUs
	set tableheader [string range $table 0 7]

	# split table of LUs
	for {set l 1} {$l <= $::menus($vts,numlus)} {incr l} {
		set idx [expr $l*8]
		set luheader($l) [string range $table $idx [expr $idx+7]]
		set startbyte($l) [get_int luheader($l) 4 4]
		set endbyte($l)   [get_int table [expr $startbyte($l)+4] 4]
		set lutable($l) [string range $table $startbyte($l) [expr $startbyte($l)+$endbyte($l)]]
	}

	# create new entry for the new LU
	set luheader($newlu) $luheader($lu)
	set lutable($newlu)  $lutable($lu)
	set startbyte($newlu) [expr $startbyte($lu)+8]

	# concatenate the tables again, adding the new LU
	set_int tableheader 0 $newlu 2
	set table $tableheader

	for {set l 1} {$l <= $newlu} {incr l} {
		set_int luheader($l) 4 [expr $startbyte($l)+8] 4
		set table $table$luheader($l)
	}
	for {set l 1} {$l <= $newlu} {incr l} {
		set table $table$lutable($l)
	}
	set_int table 4 [expr [string length $table]-1] 4
	set ::raw_sectors($vts,$tablename) $table

	# increment number of LUs
	set ::menus($vts,numlus) $newlu
	if {$newlu == 2} {
		set menus($vts,lus_are_identical) true
	}

	# refresh GUI
	::main::dvd_modified true false
	::main::init_tree_menu_no_move
	::main::display_pgc menu $vts $newlu 1

	# change the language code of the new LU
	set_lang_code
}


}	;# end of namespace ::plugins::lu
